/*
 * Copyright (C) Sergey P. Derevyago, 2008-2009.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Defines class @link ders::cycl_buf cycl_buf @endlink.
 */

#ifndef __DERS__CYCL_BUF_HPP__
#define __DERS__CYCL_BUF_HPP__

#include <ders/config.hpp>
#include <ders/sh_ptr.hpp>

namespace ders {  // ::ders

/**
 * Cyclic buffer.
 */
class cycl_buf {
      char* buf;
      int sz;
      int fst, cnt;

      cycl_buf(const cycl_buf&);
      cycl_buf& operator=(const cycl_buf&);

 public:
      cycl_buf(int rsrv);
      ~cycl_buf() { operator delete(buf); }

      int size() const { return cnt; }
      void reserve(int n);

      void put(const void* ptr, int n);
      void put(const cycl_buf& cb);
      void put_front(const void* ptr, int n);
      void put_front(const cycl_buf& cb);

      void get(void* ptr, int n);
};

typedef sh_ptr<cycl_buf> sh_cycl_buf;

/**
 * Creates new cycl_buf object.
 * @param mp mem_pool to use.
 * @param rsrv initial buffer size.
 * @return created object.
 */
sh_cycl_buf new_cycl_buf(mem_pool& mp, int rsrv);

}  // namespace ::ders

#endif  // __DERS__CYCL_BUF_HPP__

