/*
 * Copyright (C) Sergey P. Derevyago, 2008-2009.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Defines interfaces @link ders::object_io object_io @endlink, @link
 * ders::data_queue data_queue @endlink and
 * overloaded functions new_data_queue().
 */

#ifndef __DERS__DATA_QUEUE_HPP__
#define __DERS__DATA_QUEUE_HPP__

#include <ders/config.hpp>
#include <ders/data_io.hpp>

namespace ders {  // ::ders

/**
 * Interface that is used by data_queue for object I/O.
 */
class object_io : virtual public destroyable {
 public:
      virtual bool read_from(data_in& in)=0;
      virtual void write_to(data_out& out) const=0;
};

/**
 * Interface that defines interruptible data queue.
 */
class data_queue : virtual public destroyable {
 public:
      /** read/write result */
      enum result { ok, empt, intr };

      virtual bool is_intr()=0;
      virtual void set_intr(bool val)=0;

      virtual result read_obj(object_io& rdr, bool wait=true)=0;
      virtual result write_obj(const object_io& wrtr)=0;
};

typedef sh_ptr<data_queue> sh_data_queue;

/**
 * Creates new data_queue object.
 * @param mp mem_pool to use.
 * @param rsrv initial queue buffer size.
 * @return created object.
 */
sh_data_queue new_data_queue(mem_pool& mp, int rsrv=10*1024);

/**
 * Creates new data_queue object.
 * @param dio data_io to use.
 * @return created object.
 */
sh_data_queue new_data_queue(sh_data_io dio);

}  // namespace ::ders

#endif  // __DERS__DATA_QUEUE_HPP__

