/*
 * Copyright (C) Sergey P. Derevyago, 2008-2009.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Defines interface @link ders::dir dir @endlink and
 * function new_dir() and
 * set of functions for directory handling.
 */

#ifndef __DERS__DIR_HPP__
#define __DERS__DIR_HPP__

#include <ders/config.hpp>
#include <ders/error.hpp>

namespace ders {  // ::ders

/**
 * Interface that defines directory to find files in.
 */
class dir : virtual public destroyable {
 public:
      /** found directory %entry */
      struct entry {
             sh_text name;
             bool isdir;
             unsigned long size;

             entry(mem_pool& mp) : name(nt(mp)), isdir(false), size(0) {}
      };

      virtual bool find_next(entry& dent)=0;
      virtual sh_text full_name(const entry& dent)=0;
};

typedef sh_ptr<dir> sh_dir;

/**
 * Creates new dir object.
 * @param mp mem_pool to use.
 * @param dname directory name.
 * @return created object.
 */
sh_dir new_dir(mem_pool& mp, const ch_rng& dname);

error make_dir(mem_pool& mp, const ch_rng& dname);
error make_dirs(mem_pool& mp, const ch_rng& dname);

void ex_make_dir(mem_pool& mp, const ch_rng& dname);
void ex_make_dirs(mem_pool& mp, const ch_rng& dname);

}  // namespace ::ders

#endif  // __DERS__DIR_HPP__

