/*
 * Copyright (C) Sergey P. Derevyago, 2008-2009.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Defines classes @link ders::mem_pool mem_pool @endlink, @link ders::mp_newbuf
 * mp_newbuf @endlink.
 */

#ifndef __DERS__MEM_POOL_HPP__
#define __DERS__MEM_POOL_HPP__

#include <ders/config.hpp>
#include <new>
#include <stddef.h>

namespace ders {  // ::ders

/**
 * Non-synchronized memory pool to be used as thread-private object.
 */
class mem_pool {
      void* impl;

      mem_pool(const mem_pool&);
      mem_pool& operator=(const mem_pool&);

 public:
      mem_pool(size_t max_chunk_size=2*1024, size_t min_objs_in_chunk=4);
      ~mem_pool();

      void* allocate(size_t size);
      void deallocate(void* ptr, size_t size);
};

/**
 * Helper class for exception-safe memory allocation.
 */
template<class T>
class mp_newbuf {
      mem_pool& mp;
      void* ptr;

      mp_newbuf(const mp_newbuf&);
      mp_newbuf& operator=(const mp_newbuf&);

 public:
      explicit mp_newbuf(mem_pool& m) : mp(m), ptr(m.allocate(sizeof(T))) {}
      ~mp_newbuf() { if (ptr) mp.deallocate(ptr, sizeof(T)); }

      mem_pool& pool() const { return mp; }
      void* get() const { return ptr; }
      T* rls(T* p) { ptr=0; return p; }
};

}  // namespace ::ders

#endif  // __DERS__MEM_POOL_HPP__

