/*
 * Copyright (C) Sergey P. Derevyago, 2008-2009.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Defines interfaces @link ders::task_opers task_opers @endlink, @link
 * ders::task task @endlink.
 */

#ifndef __DERS__TASK_HPP__
#define __DERS__TASK_HPP__

#include <ders/config.hpp>
#include <vector>
#include <ders/data_queue.hpp>

namespace ders {  // ::ders

typedef std::vector<data_queue*> dq_vec;

/**
 * Interface that defines available task operations.
 */
class task_opers : virtual public destroyable {
 public:
      virtual void invoke(void (*fun)(void*), void* arg)=0;
};

/**
 * Interface for classes that can be executed by thread_pool.
 */
class task : virtual public destroyable {
 public:
      virtual void* proc_arg() { return 0; }
      virtual void proc(mem_pool& mp, const dq_vec& dqv, void* arg, task_opers&
        to)=0;
};

}  // namespace ::ders

#endif  // __DERS__TASK_HPP__

