/*
 * Copyright (C) Sergey P. Derevyago, 2008-2009.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Defines class @link ders::text_buf text_buf @endlink.
 */

#ifndef __DERS__TEXT_BUF_HPP__
#define __DERS__TEXT_BUF_HPP__

#include <ders/config.hpp>
#include <ders/text.hpp>

namespace ders {  // ::ders

/**
 * Helper class for %text concatenation and built-in types conversion.
 */
class text_buf {
      sh_text txt;

      text_buf(const text_buf&);
      text_buf& operator=(const text_buf&);

 public:
      explicit text_buf(mem_pool& mp, int rsv=1024);
      explicit text_buf(sh_text sht, int rsv=1024);

      text_buf& operator+(const ch_rng& r);
      text_buf& operator+(char byte);

      text_buf& operator+(bool val);
      text_buf& operator+(int val);
      text_buf& operator+(unsigned val);
      text_buf& operator+(long val);
      text_buf& operator+(unsigned long val);

      text_buf& operator+(const char* str);
      text_buf& operator+(void* ptr);

      operator const sh_text&() const { return txt; }
      operator ch_rng() const { return ch_rng(txt); }

      int size() const { return txt->size(); }
};

inline text_buf::text_buf(mem_pool& mp, int rsv) : txt(nt(mp))
{ txt->reserve(rsv); }

inline text_buf::text_buf(sh_text sht, int rsv) : txt(nt(sht.pool(), sht))
{ txt->reserve(rsv); }

inline text_buf& text_buf::operator+(const ch_rng& r)
{
 txt->append(r);
 return *this;
}

inline text_buf& text_buf::operator+(char byte)
{
 txt->append(byte);
 return *this;
}

inline text_buf& text_buf::operator+(bool val)
{ return *this+(val? "true" : "false"); }

inline text_buf& text_buf::operator+(const char* str)
{ return *this+ch_rng(str); }

}  // namespace ::ders

#endif  // __DERS__TEXT_BUF_HPP__

