/*
 * Copyright (C) Sergey P. Derevyago, 2008-2009.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Defines interface @link ders::thread_pool thread_pool @endlink and
 * overloaded functions new_thread_pool().
 */

#ifndef __DERS__THREAD_POOL_HPP__
#define __DERS__THREAD_POOL_HPP__

#include <ders/config.hpp>
#include <ders/task.hpp>

namespace ders {  // ::ders

/**
 * Interface that defines %thread pool. The threads in the pool execute passed
 * task together with the caller thread.
 */
class thread_pool : virtual public destroyable {
 public:
      virtual void exec(task& tk, const dq_vec& dv)=0;
};

typedef sh_ptr<thread_pool> sh_thread_pool;

/**
 * Creates new singlethreaded thread_pool object.
 * @param mp mem_pool to use.
 * @return created object.
 */
sh_thread_pool new_thread_pool(mem_pool& mp);

/**
 * Creates new multithreaded thread_pool object.
 * @param mp mem_pool to use.
 * @param cnt threads count.
 * @return created object.
 */
sh_thread_pool new_thread_pool(mem_pool& mp, int cnt);

}  // namespace ::ders

#endif  // __DERS__THREAD_POOL_HPP__

