/*
 * Copyright (C) Sergey P. Derevyago, 2008-2009.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Defines class @link ders::wldcrd_mtchr wldcrd_mtchr @endlink.
 */

#ifndef __DERS__WLDCRD_MTCHR_HPP__
#define __DERS__WLDCRD_MTCHR_HPP__

#include <ders/config.hpp>
#include <list>
#include <vector>
#include <ders/stl_alloc.hpp>
#include <ders/text.hpp>

namespace ders {  // ::ders

/**
 * Represents implementation independent %file name wildcard matcher.
 */
class wldcrd_mtchr {
      struct word {
             int off;
             ch_rng str;

             word(int o, const ch_rng& s) : off(o), str(s) {}
      };

      struct segm {
             int len;
             std::vector<word> wds;

             segm() : len(0) {}
      };

      mem_pool& mp;
      int len;
      std::vector<segm*> sgs;
      sh_text sav_msk;
      std::list<segm, stl_allocator<segm> > sav_sgs;

      wldcrd_mtchr(const wldcrd_mtchr&);
      wldcrd_mtchr& operator=(const wldcrd_mtchr&);

      void add_sg();
      bool eq(const char* str, const segm& sg, int beg=0) const;
      const char* fnd(const ch_rng& str, const segm& sg) const;

 public:
      wldcrd_mtchr(mem_pool& m, const ch_rng& mask);

      const ch_rng mask() const { return sav_msk; }
      bool match(const ch_rng& str) const;
};

typedef sh_ptr<wldcrd_mtchr> sh_wldcrd_mtchr;

/**
 * Creates new wldcrd_mtchr object.
 * @param mp mem_pool to use.
 * @param mask mask to match with.
 * @return created object.
 */
sh_wldcrd_mtchr new_wldcrd_mtchr(mem_pool& mp, const ch_rng& mask);

}  // namespace ::ders

#endif  // __DERS__WLDCRD_MTCHR_HPP__

