/*
 * Copyright (C) Sergey P. Derevyago, 2008-2009.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Implements class @link ders::cycl_buf cycl_buf @endlink.
 */

#include <ders/cycl_buf.hpp>
#include <assert.h>
#include <string.h>

namespace ders {  // ::ders

cycl_buf::cycl_buf(int rsrv)
{
 assert(rsrv>0);
 sz=rsrv;
 buf=static_cast<char*>(operator new(sz));
 fst=0;
 cnt=0;
}

void cycl_buf::reserve(int n)
{
 assert(n>=0);
 int nsz=cnt+n;
 if (nsz<=sz) return;

 if (nsz<sz*2) nsz=sz*2;
 char* nbuf=static_cast<char*>(operator new(nsz));

 int n1=sz-fst;
 if (cnt<=n1) memcpy(nbuf, buf+fst, cnt);
 else {
      memcpy(nbuf, buf+fst, n1);
      memcpy(nbuf+n1, buf, cnt-n1);
 }

 operator delete(buf);
 buf=nbuf;
 sz=nsz;
 fst=0;
}

void cycl_buf::put(const void* ptr, int n)
{
 assert(n>=0);
 if (cnt+n>sz) reserve(cnt+n);

 int pos=(fst+cnt)%sz, n1=sz-pos;
 if (n<=n1) memcpy(buf+pos, ptr, n);
 else {
      memcpy(buf+pos, ptr, n1);
      memcpy(buf, static_cast<const char*>(ptr)+n1, n-n1);
 }

 cnt+=n;
}

void cycl_buf::put(const cycl_buf& cb)
{
 if (cnt+cb.cnt>sz) reserve(cnt+cb.cnt);

 if (cb.fst+cb.cnt<=cb.sz) put(cb.buf+cb.fst, cb.cnt);
 else {
      put(cb.buf+cb.fst, cb.sz-cb.fst);
      put(cb.buf, cb.fst+cb.cnt-cb.sz);
 }
}

void cycl_buf::put_front(const void* ptr, int n)
{
 assert(n>=0);
 if (cnt+n>sz) reserve(cnt+n);

 int nfst=fst-n;
 if (nfst>=0) {
    fst=nfst;
    memcpy(buf+fst, ptr, n);
 }
 else {
      fst=sz+nfst;
      memcpy(buf+fst, ptr, -nfst);
      memcpy(buf, static_cast<const char*>(ptr)-nfst, n+nfst);
 }

 cnt+=n;
}

void cycl_buf::put_front(const cycl_buf& cb)
{
 if (cnt+cb.cnt>sz) reserve(cnt+cb.cnt);

 if (cb.fst+cb.cnt<=cb.sz) put_front(cb.buf+cb.fst, cb.cnt);
 else {
      put_front(cb.buf, cb.fst+cb.cnt-cb.sz);
      put_front(cb.buf+cb.fst, cb.sz-cb.fst);
 }
}

void cycl_buf::get(void* ptr, int n)
{
 assert(n<=cnt && n>=0);

 int n1=sz-fst;
 if (n<=n1) memcpy(ptr, buf+fst, n);
 else {
      memcpy(ptr, buf+fst, n1);
      memcpy(static_cast<char*>(ptr)+n1, buf, n-n1);
 }

 fst=(fst+n)%sz;
 cnt-=n;
}

sh_cycl_buf new_cycl_buf(mem_pool& mp, int rsrv)
{
 mp_newbuf<cycl_buf> buf(mp);
 return sh_cycl_buf(buf.pool(), buf.rls(::new(buf.get()) cycl_buf(rsrv)));
}

}  // namespace ::ders

