/*
 * Copyright (C) Sergey P. Derevyago, 2008.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Implements overloaded functions new_data_queue().
 */

#include <ders/data_queue.hpp>
#include <ders/destroy.hpp>

namespace {

using namespace ders;

class dq_impl : public data_queue {
      bool ir;
      sh_data_io dio;

 public:
      dq_impl(sh_data_io d) : ir(false), dio(d) {}

      virtual void destroy(mem_pool& mp2) { destroy_this(this, mp2); }
      virtual bool is_intr();
      virtual void set_intr(bool val);
      virtual result read_obj(object_io& rdr, bool wait);
      virtual result write_obj(const object_io& wrtr);
};

bool dq_impl::is_intr()
{
 return ir;
}

void dq_impl::set_intr(bool val)
{
 ir=val;
}

data_queue::result dq_impl::read_obj(object_io& rdr, bool)
{
 if (ir) return intr;
 return (rdr.read_from(*dio)) ? ok : empt;
}

data_queue::result dq_impl::write_obj(const object_io& wrtr)
{
 if (ir) return intr;

 wrtr.write_to(*dio);
 return ok;
}

}  // unnamed

namespace ders {  // ::ders

sh_data_queue new_data_queue(mem_pool& mp, int rsrv)
{
 return new_data_queue(new_data_io(new_cycl_buf(mp, rsrv)));
}

sh_data_queue new_data_queue(sh_data_io dio)
{
 mp_newbuf<dq_impl> buf(dio.pool());
 return sh_data_queue(buf.pool(), buf.rls(::new(buf.get()) dq_impl(dio)));
}

}  // namespace ::ders

