/*
 * Copyright (C) Sergey P. Derevyago, 2008.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Implements set of functions for directory handling.
 */

#include <ders/dir.hpp>
#include <ders/file.hpp>
#include <ders/path.hpp>

namespace ders {  // ::ders

using namespace std;

error make_dirs(mem_pool& mp, const ch_rng& dname)
{
 vector<path_elmt> vpe;
 parse_path(mp, dname, vpe);

 int i=0, end=vpe.size();
 for ( ; ; i++) {
     if (i>=end) return einval;
     if (vpe[i].tp==path_elmt::name) break;
 }

 error err;
 do {
    err=make_dir(mp, make_path(mp, &vpe[0], &vpe[0]+i+1));
    if (err!=ezero && err!=eexist) return err;
 } while (++i<end);

 return err;
}

void ex_make_dir(mem_pool& mp, const ch_rng& dname)
{
 error err=make_dir(mp, dname);
 if (err!=ezero) {
    throw newFileErrorException(mp, _FLINE_, "Can't make directory", err,
      dname);
 }
}

void ex_make_dirs(mem_pool& mp, const ch_rng& dname)
{
 error err=make_dirs(mp, dname);
 if (err!=ezero) {
    throw newFileErrorException(mp, _FLINE_, "Can't make directories", err,
      dname);
 }
}

}  // namespace ::ders

