/*
 * Copyright (C) Sergey P. Derevyago, 2008.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Implements set of functions for memory searching.
 */

#include <ders/mem.hpp>
#include <string.h>

namespace ders {  // ::ders

void* mem_find(void* p1, int sz1, char ch)
{
 return (sz1>0) ? memchr(p1, ch, sz1) : 0;
}

void* mem_rev_find(void* p1, int sz1, char ch)
{
 char *e=(char*)p1+sz1-1, *b=e-sz1;

 for ( ; e>=b+4; e-=4) {
     if (*(e-0)==ch) return e-0;
     if (*(e-1)==ch) return e-1;
     if (*(e-2)==ch) return e-2;
     if (*(e-3)==ch) return e-3;
 }

 switch (e-b) {
        case 3:
             if (*(e-0)==ch) return e-0;
             if (*(e-1)==ch) return e-1;
             if (*(e-2)==ch) return e-2;
             break;
        case 2:
             if (*(e-0)==ch) return e-0;
             if (*(e-1)==ch) return e-1;
             break;
        case 1:
             if (*(e-0)==ch) return e-0;
             break;
 }

 return 0;
}

void* mem_find(void* p1, int sz1, const void* p2, int sz2)
{
 if (sz1<=0 || sz2<=0) return 0;

 for (char *b=(char*)p1, *e=b+sz1-sz2+1; b<e; ) {
     char* ptr=(char*)mem_find(b, e-b, *(char*)p2);
     if (!ptr) return 0;

     if (memcmp(ptr+1, (char*)p2+1, sz2-1)==0) return ptr;
     b=ptr+1;
 }

 return 0;
}

void* mem_rev_find(void* p1, int sz1, const void* p2, int sz2)
{
 if (sz1<=0 || sz2<=0) return 0;

 for (char *b=(char*)p1, *e=b+sz1-sz2+1; b<e; ) {
     char* ptr=(char*)mem_rev_find(b, e-b, *(char*)p2);
     if (!ptr) return 0;

     if (memcmp(ptr+1, (char*)p2+1, sz2-1)==0) return ptr;
     e=ptr;
 }

 return 0;
}

}  // namespace ::ders

