/*
 * Copyright (C) Sergey P. Derevyago, 2008.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Win32 implementation of functions new_dir(), make_dir().
 */

#include <windows.h>
#include <ders/dir.hpp>
#include <ders/file.hpp>
#include <ders/text_buf.hpp>

namespace {

using namespace ders;

error errTbl[]={
  ezero,      //    0  ERROR_SUCCESS
  einval,     //    1  ERROR_INVALID_FUNCTION
  enoent,     //    2  ERROR_FILE_NOT_FOUND
  enoent,     //    3  ERROR_PATH_NOT_FOUND
  emfile,     //    4  ERROR_TOO_MANY_OPEN_FILES
  eacces,     //    5  ERROR_ACCESS_DENIED
  ebadf,      //    6  ERROR_INVALID_HANDLE
  enomem,     //    7  ERROR_ARENA_TRASHED
  enomem,     //    8  ERROR_NOT_ENOUGH_MEMORY
  enomem,     //    9  ERROR_INVALID_BLOCK
  e2big,      //   10  ERROR_BAD_ENVIRONMENT
  enoexec,    //   11  ERROR_BAD_FORMAT
  einval,     //   12  ERROR_INVALID_ACCESS
  einval,     //   13  ERROR_INVALID_DATA
  enomem,     //   14  ERROR_OUTOFMEMORY
  enoent,     //   15  ERROR_INVALID_DRIVE
  eacces,     //   16  ERROR_CURRENT_DIRECTORY
  exdev,      //   17  ERROR_NOT_SAME_DEVICE
  enoent,     //   18  ERROR_NO_MORE_FILES
  erofs,      //   19  ERROR_WRITE_PROTECT
  enxio,      //   20  ERROR_BAD_UNIT
  ebusy,      //   21  ERROR_NOT_READY
  eio,        //   22  ERROR_BAD_COMMAND
  eio,        //   23  ERROR_CRC
  eio,        //   24  ERROR_BAD_LENGTH
  eio,        //   25  ERROR_SEEK
  eio,        //   26  ERROR_NOT_DOS_DISK
  enxio,      //   27  ERROR_SECTOR_NOT_FOUND
  ebusy,      //   28  ERROR_OUT_OF_PAPER
  eio,        //   29  ERROR_WRITE_FAULT
  eio,        //   30  ERROR_READ_FAULT
  eio,        //   31  ERROR_GEN_FAILURE
  eacces,     //   32  ERROR_SHARING_VIOLATION
  eacces,     //   33  ERROR_LOCK_VIOLATION
  enxio,      //   34  ERROR_WRONG_DISK
  enfile,     //   35  ERROR_FCB_UNAVAILABLE
  enfile,     //   36  ERROR_SHARING_BUFFER_EXCEEDED
  efault,     //   37  ERROR_CODE_PAGE_MISMATCHED
  eio,        //   38  ERROR_HANDLE_EOF
  enospc,     //   39  ERROR_HANDLE_DISK_FULL
  eunkn,      //   40
  eunkn,      //   41
  eunkn,      //   42
  eunkn,      //   43
  eunkn,      //   44
  eunkn,      //   45
  eunkn,      //   46
  eunkn,      //   47
  eunkn,      //   48
  eunkn,      //   49
  enodev,     //   50  ERROR_NOT_SUPPORTED
  ebusy,      //   51  ERROR_REM_NOT_LIST
  eexist,     //   52  ERROR_DUP_NAME
  enoent,     //   53  ERROR_BAD_NETPATH
  ebusy,      //   54  ERROR_NETWORK_BUSY
  enodev,     //   55  ERROR_DEV_NOT_EXIST
  eagain,     //   56  ERROR_TOO_MANY_CMDS
  eio,        //   57  ERROR_ADAP_HDW_ERR
  eio,        //   58  ERROR_BAD_NET_RESP
  eio,        //   59  ERROR_UNEXP_NET_ERR
  einval,     //   60  ERROR_BAD_REM_ADAP
  efbig,      //   61  ERROR_PRINTQ_FULL
  enospc,     //   62  ERROR_NO_SPOOL_SPACE
  enoent,     //   63  ERROR_PRINT_CANCELLED
  enoent,     //   64  ERROR_NETNAME_DELETED
  eacces,     //   65  ERROR_NETWORK_ACCESS_DENIED
  enodev,     //   66  ERROR_BAD_DEV_TYPE
  enoent,     //   67  ERROR_BAD_NET_NAME
  enfile,     //   68  ERROR_TOO_MANY_NAMES
  eio,        //   69  ERROR_TOO_MANY_SESS
  eagain,     //   70  ERROR_SHARING_PAUSED
  einval,     //   71  ERROR_REQ_NOT_ACCEP
  eagain,     //   72  ERROR_REDIR_PAUSED
  efault,     //   73  ERROR_SBCS_ATT_WRITE_PROT
  efault,     //   74  ERROR_SBCS_GENERAL_FAILURE
  efault,     //   75  ERROR_XGA_OUT_MEMORY
  eunkn,      //   76
  eunkn,      //   77
  eunkn,      //   78
  eunkn,      //   79
  eexist,     //   80  ERROR_FILE_EXISTS
  efault,     //   81  ERROR_DUP_FCB
  eacces,     //   82  ERROR_CANNOT_MAKE
  eacces,     //   83  ERROR_FAIL_I24
  enfile,     //   84  ERROR_OUT_OF_STRUCTURES
  eexist,     //   85  ERROR_ALREADY_ASSIGNED
  eperm,      //   86  ERROR_INVALID_PASSWORD
  einval,     //   87  ERROR_INVALID_PARAMETER
  eio,        //   88  ERROR_NET_WRITE_FAULT
  eagain,     //   89  ERROR_NO_PROC_SLOTS
  eunkn,      //   90  ERROR_NOT_FROZEN
  eunkn,      //   91  ERR_TSTOVFL
  eunkn,      //   92  ERR_TSTDUP
  eunkn,      //   93  ERROR_NO_ITEMS
  eunkn,      //   94
  eintr,      //   95  ERROR_INTERRUPT
  eunkn,      //   96
  eunkn,      //   97
  eunkn,      //   98
  ebusy,      //   99  ERROR_DEVICE_IN_USE
  eagain,     //  100  ERROR_TOO_MANY_SEMAPHORES
  eagain,     //  101  ERROR_EXCL_SEM_ALREADY_OWNED
  eunkn,      //  102  ERROR_SEM_IS_SET
  eunkn,      //  103  ERROR_TOO_MANY_SEM_REQUESTS
  eunkn,      //  104  ERROR_INVALID_AT_INTERRUPT_TIME
  eunkn,      //  105  ERROR_SEM_OWNER_DIED
  eunkn,      //  106  ERROR_SEM_USER_LIMIT
  exdev,      //  107  ERROR_DISK_CHANGE
  eacces,     //  108  ERROR_DRIVE_LOCKED
  epipe,      //  109  ERROR_BROKEN_PIPE
  enoent,     //  110  ERROR_OPEN_FAILED
  eunkn,      //  111  ERROR_BUFFER_OVERFLOW
  enospc,     //  112  ERROR_DISK_FULL
  emfile,     //  113  ERROR_NO_MORE_SEARCH_HANDLES
  ebadf,      //  114  ERROR_INVALID_TARGET_HANDLE
  efault,     //  115  ERROR_PROTECTION_VIOLATION
  eunkn,      //  116  ERROR_VIOKBD_REQUEST
  eunkn,      //  117  ERROR_INVALID_CATEGORY
  eunkn,      //  118  ERROR_INVALID_VERIFY_SWITCH
  eunkn,      //  119  ERROR_BAD_DRIVER_LEVEL
  eunkn,      //  120  ERROR_CALL_NOT_IMPLEMENTED
  eunkn,      //  121  ERROR_SEM_TIMEOUT
  eunkn,      //  122  ERROR_INSUFFICIENT_BUFFER
  enoent,     //  123  ERROR_INVALID_NAME
  einval,     //  124  ERROR_INVALID_LEVEL
  eunkn,      //  125  ERROR_NO_VOLUME_LABEL
  eunkn,      //  126  ERROR_MOD_NOT_FOUND
  esrch,      //  127  ERROR_PROC_NOT_FOUND
  echild,     //  128  ERROR_WAIT_NO_CHILDREN
  echild,     //  129  ERROR_CHILD_NOT_COMPLETE
  ebadf,      //  130  ERROR_DIRECT_ACCESS_HANDLE
  einval,     //  131  ERROR_NEGATIVE_SEEK
  eacces,     //  132  ERROR_SEEK_ON_DEVICE
  eunkn,      //  133  ERROR_IS_JOIN_TARGET
  eunkn,      //  134  ERROR_IS_JOINED
  eunkn,      //  135  ERROR_IS_SUBSTED
  eunkn,      //  136  ERROR_NOT_JOINED
  eunkn,      //  137  ERROR_NOT_SUBSTED
  eunkn,      //  138  ERROR_JOIN_TO_JOIN
  eunkn,      //  139  ERROR_SUBST_TO_SUBST
  eunkn,      //  140  ERROR_JOIN_TO_SUBST
  eunkn,      //  141  ERROR_SUBST_TO_JOIN
  eagain,     //  142  ERROR_BUSY_DRIVE
  eunkn,      //  143  ERROR_SAME_DRIVE
  eunkn,      //  144  ERROR_DIR_NOT_ROOT
  enotempty,  //  145  ERROR_DIR_NOT_EMPTY
  eunkn,      //  146  ERROR_IS_SUBST_PATH
  eunkn,      //  147  ERROR_IS_JOIN_PATH
  eunkn,      //  148  ERROR_PATH_BUSY
  eunkn,      //  149  ERROR_IS_SUBST_TARGET
  eunkn,      //  150  ERROR_SYSTEM_TRACE
  eunkn,      //  151  ERROR_INVALID_EVENT_COUNT
  eunkn,      //  152  ERROR_TOO_MANY_MUXWAITERS
  eunkn,      //  153  ERROR_INVALID_LIST_FORMAT
  eunkn,      //  154  ERROR_LABEL_TOO_LONG
  eunkn,      //  155  ERROR_TOO_MANY_TCBS
  eunkn,      //  156  ERROR_SIGNAL_REFUSED
  eunkn,      //  157  ERROR_DISCARDED
  eacces,     //  158  ERROR_NOT_LOCKED
  eunkn,      //  159  ERROR_BAD_THREADID_ADDR
  eunkn,      //  160  ERROR_BAD_ARGUMENTS
  enoent,     //  161  ERROR_BAD_PATHNAME
  eunkn,      //  162  ERROR_SIGNAL_PENDING
  eunkn,      //  163  ERROR_UNCERTAIN_MEDIA
  eagain,     //  164  ERROR_MAX_THRDS_REACHED
  eunkn,      //  165  ERROR_MONITORS_NOT_SUPPORTED
  eunkn,      //  166  ERROR_UNC_DRIVER_NOT_INSTALLED
  eacces,     //  167  ERROR_LOCK_FAILED
  eunkn,      //  168  ERROR_SWAPIO_FAILED
  eunkn,      //  169  ERROR_SWAPIN_FAILED
  eunkn,      //  170  ERROR_BUSY
  eunkn,      //  171
  eunkn,      //  172
  eunkn,      //  173  ERROR_CANCEL_VIOLATION
  eunkn,      //  174  ERROR_ATOMIC_LOCK_NOT_SUPPORTED
  eunkn,      //  175  ERROR_READ_LOCKS_NOT_SUPPORTED
  eunkn,      //  176
  eunkn,      //  177
  eunkn,      //  178
  eunkn,      //  179
  eunkn,      //  180  ERROR_INVALID_SEGMENT_NUMBER
  eunkn,      //  181  ERROR_INVALID_CALLGATE
  eunkn,      //  182  ERROR_INVALID_ORDINAL
  eexist,     //  183  ERROR_ALREADY_EXISTS
  echild,     //  184  ERROR_NO_CHILD_PROCESS
  eunkn,      //  185  ERROR_CHILD_ALIVE_NOWAIT
  eunkn,      //  186  ERROR_INVALID_FLAG_NUMBER
  eunkn,      //  187  ERROR_SEM_NOT_FOUND
  eunkn,      //  188  ERROR_INVALID_STARTING_CODESEG
  eunkn,      //  189  ERROR_INVALID_STACKSEG
  eunkn,      //  190  ERROR_INVALID_MODULETYPE
  eunkn,      //  191  ERROR_INVALID_EXE_SIGNATURE
  eunkn,      //  192  ERROR_EXE_MARKED_INVALID
  eunkn,      //  193  ERROR_BAD_EXE_FORMAT
  eunkn,      //  194  ERROR_ITERATED_DATA_EXCEEDS_64k
  eunkn,      //  195  ERROR_INVALID_MINALLOCSIZE
  eunkn,      //  196  ERROR_DYNLINK_FROM_INVALID_RING
  eunkn,      //  197  ERROR_IOPL_NOT_ENABLED
  eunkn,      //  198  ERROR_INVALID_SEGDPL
  eunkn,      //  199  ERROR_AUTODATASEG_EXCEEDS_64k
  eunkn,      //  200  ERROR_RING2SEG_MUST_BE_MOVABLE
  eunkn,      //  201  ERROR_RELOC_CHAIN_XEEDS_SEGLIM
  eunkn,      //  202  ERROR_INFLOOP_IN_RELOC_CHAIN
  eunkn,      //  203  ERROR_ENVVAR_NOT_FOUND
  eunkn,      //  204  ERROR_NOT_CURRENT_CTRY
  eunkn,      //  205  ERROR_NO_SIGNAL_SENT
  enoent,     //  206  ERROR_FILENAME_EXCED_RANGE
  eunkn,      //  207  ERROR_RING2_STACK_IN_USE
  eunkn,      //  208  ERROR_META_EXPANSION_TOO_LONG
  eunkn,      //  209  ERROR_INVALID_SIGNAL_NUMBER
  eunkn,      //  210  ERROR_THREAD_1_INACTIVE
  eunkn,      //  211  ERROR_INFO_NOT_AVAIL
  eunkn,      //  212  ERROR_LOCKED
  eunkn,      //  213  ERROR_BAD_DYNALINK
  eunkn,      //  214  ERROR_TOO_MANY_MODULES
  eagain,     //  215  ERROR_NESTING_NOT_ALLOWED
  eunkn,      //  216  ERROR_CANNOT_SHRINK
  eunkn,      //  217  ERROR_ZOMBIE_PROCESS
  eunkn,      //  218  ERROR_STACK_IN_HIGH_MEMORY
  eunkn,      //  219  ERROR_INVALID_EXITROUTINE_RING
  eunkn,      //  220  ERROR_GETBUF_FAILED
  eunkn,      //  221  ERROR_FLUSHBUF_FAILED
  eunkn,      //  222  ERROR_TRANSFER_TOO_LONG
  eunkn,      //  223  ERROR_FORCENOSWAP_FAILED
  eunkn,      //  224  ERROR_SMG_NO_TARGET_WINDOW
  eunkn,      //  225  ERROR_VIRUS_INFECTED
  eunkn,      //  226  ERROR_VIRUS_DELETED
  eunkn,      //  227
  eunkn,      //  228  ERROR_NO_CHILDREN
  eunkn,      //  229  ERROR_INVALID_SCREEN_GROUP
  epipe,      //  230  ERROR_BAD_PIPE
  eagain,     //  231  ERROR_PIPE_BUSY
  eunkn,      //  232  ERROR_NO_DATA
  epipe,      //  233  ERROR_PIPE_NOT_CONNECTED
};

error convertWinError(unsigned val)
{
 return (val<sizeof(errTbl)/sizeof(*errTbl)) ? errTbl[val] : eunkn;
}

shException createException(mem_pool& mp, const FileLine& location)
{
 DWORD le=GetLastError();

 char* buf;
 FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM, NULL,
   le, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (char*)&buf, 0, NULL);

 sh_text msg=text_buf(mp)+le+": "+buf;
 LocalFree(buf);

 while (msg->back()=='\r' || msg->back()=='\n')
       msg->uninitialized_resize(msg->size()-1);

 return newException(mp, location, msg);
}

struct win_dir : public dir {
       sh_text mask;
       WIN32_FIND_DATA fd;
       HANDLE hn;

       win_dir(sh_text dname);
       ~win_dir();

       virtual bool find_next(entry&);
       virtual sh_text full_name(const entry& dent);

       virtual void destroy(mem_pool& mp2) { destroy_this(this, mp2); }
};

win_dir::win_dir(sh_text dname) : mask(dname), hn(INVALID_HANDLE_VALUE)
{
 if (mask->size()>0 && mask->back()!='\\' && mask->back()!=':')
    mask->append('\\');
 mask->append('*');
}

win_dir::~win_dir()
{
 if (hn!=INVALID_HANDLE_VALUE) FindClose(hn);
}

bool win_dir::find_next(entry& dent)
{
 mem_pool& mp=mask.pool();

 if (hn==INVALID_HANDLE_VALUE) {
    hn=FindFirstFile(mask->c_str(), &fd);

    if (hn==INVALID_HANDLE_VALUE) {
       throw newFileErrorException(_FLINE_, "Can't find first file",
         convertWinError(GetLastError()), mask, createException(mp, _FLINE_));
    }
 }
 else {
      if (!FindNextFile(hn, &fd)) {
         if (GetLastError()==ERROR_NO_MORE_FILES)
            return false;

         throw newFileErrorException(_FLINE_, "Can't find next file",
           convertWinError(GetLastError()), mask, createException(mp, _FLINE_));
      }
 }

 *dent.name=fd.cFileName;
 dent.isdir=fd.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY;
 dent.size=fd.nFileSizeLow;

 return true;
}

sh_text win_dir::full_name(const entry& dent)
{
 return text_buf(mask.pool())+ch_rng(mask->begin(), mask->end()-1)+dent.name;
}

}  // unnamed

namespace ders {  // ::ders

sh_dir new_dir(mem_pool& mp, const ch_rng& dname)
{
 mp_newbuf<win_dir> buf(mp);
 return sh_dir(buf.pool(), buf.rls(::new(buf.get()) win_dir(nt(mp, dname))));
}

error make_dir(mem_pool& mp, const ch_rng& dname)
{
 return (CreateDirectory(text(mp, dname).c_str(), NULL)) ? ezero :
   convertWinError(GetLastError());
}

}  // namespace ::ders

