/*
 * Copyright (C) Sergey P. Derevyago, 2008-2009.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Win32 implementation of class @link ders::fd fd @endlink.
 */

#include <ders/file.hpp>
#include <errno.h>
#include <fcntl.h>
#include <io.h>
#include <stdio.h>
#include <sys/stat.h>

namespace {

using namespace ders;

inline int setErr(int rc, error& err)
{
 err=(rc==-1) ? convert_errno(errno) : ezero;
 return rc;
}

}  // unnamed

namespace ders {  // ::ders

int fd::open(const char* path, file::mode md, int fls, error& err)
{
 int m=0;
 switch (md) {
        case file::rdo:  m=O_RDONLY; break;
        case file::wro:  m=O_WRONLY; break;
        case file::rdwr: m=O_RDWR;   break;
        default:         assert(false);
 }

 if (fls&file::app)  m|=O_APPEND;
 if (fls&file::crt)  m|=O_CREAT;
 if (fls&file::excl) m|=O_EXCL;
 if (fls&file::trnc) m|=O_TRUNC;

 return setErr(::open(path, m|O_BINARY, S_IREAD|S_IWRITE), err);
}

bool fd::close(int fd, error& err)
{
 return setErr(::close(fd), err)==0;
}

int fd::read(int fd, void* buf, int n, error& err)
{
 assert(n>0);
 int rc=::read(fd, buf, n);
 assert(rc<=n);

 return setErr(rc, err);
}

int fd::write(int fd, const void* buf, int n, error& err)
{
 assert(n>0);
 int rc=::write(fd, buf, n);
 assert(rc<=n);

 return setErr(rc, err);
}

int fd::seek(int fd, int off, file::whence wh, error& err)
{
 int w=0;
 switch (wh) {
        case file::set: w=SEEK_SET; break;
        case file::cur: w=SEEK_CUR; break;
        case file::end: w=SEEK_END; break;
        default:        assert(false);
 }

 return setErr(::lseek(fd, off, w), err);
}

}  // namespace ::ders

