/*
 * Copyright (C) Sergey P. Derevyago, 2008.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Internal barrier test.
 */

#include <memory>
#include <vector>
#include <stdio.h>
#include <ders/thread.hpp>

using namespace std;
using namespace ders;

const int NUM_THREADS=10;
const int N=1000000;

struct MainData {
       int v[NUM_THREADS];
       sh_barrier bar;
       sh_mutex mtx;

       MainData(mem_pool& mp) :
         bar(new_barrier(mp, NUM_THREADS)), mtx(new_mutex(mp))
       { memset(v, 0, sizeof(v)); }
};

struct ThreadData {
       MainData* md;
       int n;

       ThreadData(MainData* md_, int n_) : md(md_), n(n_) {}
};

void start(void* arg)
{
 ThreadData* td=(ThreadData*)arg;
 auto_ptr<ThreadData> guard(td);

 struct blnr : barrier_listener {
        ThreadData* td;
        int n;

        blnr(ThreadData* t) : td(t), n(0) {}

        virtual barrier_listener::action on_release(barrier& bar)
        {
         if (++n*(td->n)%7==0) return barrier_listener::release_one_false;
         if (++n*(td->n)%11==0) return barrier_listener::release_all_false;

         return barrier_listener::release_all_true;
        }

        virtual void destroy(mem_pool&) { hard_assert(false); }
 } bl(td);

 for (int i=0; i<N; i++) {
     td->md->v[td->n]++;

     if ((i+td->n)%(NUM_THREADS*3*10)==0) td->md->bar->release_one();
     if ((i+td->n)%(NUM_THREADS*3*10)==1) td->md->bar->release_all();

     while (!td->md->bar->wait())
           ;
 }
}

int main()
{
 mem_pool mp;
 MainData mdo(mp), *md=&mdo;

 vector<sh_thread> vthr;
 for (int i=0; i<NUM_THREADS; i++)
     vthr.push_back(new_thread(mp, start, new ThreadData(md, i)));

 for (int i=0; i<NUM_THREADS; i++) vthr[i]->join();

 for (int i=0; i<NUM_THREADS; i++) {
     printf("%d ", md->v[i]);
     hard_assert(md->v[i]==md->v[0]);
 }
}

