/*
 * Copyright (C) Sergey P. Derevyago, 2008.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Internal dir test.
 */

#include <ders/dir.hpp>
#include <ders/file.hpp>
#include <ders/text_buf.hpp>

using namespace ders;

int main(int argc, char** argv)
{
 mem_pool mp;
 file out(mp, fd::out);
 file err(mp, fd::err);

 shException error(mp, 0);
 try {
     if (argc!=2) {
        err.write("tst_dir dir\n");
        return 1;
     }

     sh_dir dr=new_dir(mp, argv[1]);
     for (dir::entry dent(mp); dr->find_next(dent); ) {
         out.write(text_buf(mp)+dent.isdir+'\t'+dent.size+'\t'+
           dr->full_name(dent)+'\n');
     }

     return 0;
 }
 catch (shException she) { error=she; }
 catch (...) { error=recatchException(mp, _FLINE_); }

 err.write(toTextAll(error));
 return 1;
}

