/*
 * Copyright (C) Sergey P. Derevyago, 2008.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Internal hash_vec test.
 */

#include <map>
#include <stdio.h>
#include <time.h>
#include <ders/file.hpp>
#include <ders/hash_vec.hpp>
#include <ders/text_buf.hpp>

using namespace std;
using namespace ders;

const int N=10000;
const int M=1000;

void f_std()
{
 typedef map<sh_text, size_t> map_type;

 mem_pool mp;
 map_type m;
 char buf[10];

 for (int i=0; i<M; i++) {
     sprintf(buf, "%d", i);
     m.insert(map_type::value_type(nt(mp, buf), i));
 }
 for (int i=0; i<M; i++) {
     sprintf(buf, "%d", i);
     m.erase(nt(mp, buf));
 }
}

void f_ders()
{
 mem_pool mp;
 hash_vec<sh_text, size_t> ht(M);
 char buf[10];

 for (int i=0; i<M; i++) {
     sprintf(buf, "%d", i);
     ht.insert(nt(mp, buf), i);
 }
 for (int i=0; i<M; i++) {
     sprintf(buf, "%d", i);
     ht.erase(ht.find(nt(mp, buf)));
 }
}

int main(int argc, char** argv)
{
 if (argc!=2) {
    m1:
    fprintf(stderr, "tst_hash std|ders");
    return 1;
 }

 void (*f)();
 if (strcmp(argv[1], "std")==0) f=f_std;
 else if (strcmp(argv[1], "ders")==0) f=f_ders;
 else goto m1;

 clock_t c1=clock();

 for (int i=0; i<N; i++) f();

 clock_t c2=clock();
 printf("%d\t%s\n", int(c2-c1), argv[1]);

 return 0;
}

