/*
 * Copyright (C) Sergey P. Derevyago, 2008.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Internal parse_path() test.
 */

#include <ders/file.hpp>
#include <ders/path.hpp>
#include <ders/text_buf.hpp>

using namespace std;
using namespace ders;

void prn(file& f, const vector<path_elmt>& vpe)
{
 for (int i=0, end=vpe.size(); i<end; i++) {
     const char* strtp=0;
     switch (vpe[i].tp) {
            case path_elmt::drive: strtp="drive"; break;
            case path_elmt::name:  strtp="name";  break;
            case path_elmt::root:  strtp="root";  break;
            case path_elmt::share: strtp="share"; break;
     }

     f.write(text_buf(f.pool())+strtp+"("+vpe[i].nm+")\n");
 }
}

sh_text cnv(mem_pool& mp, const char* path)
{
 vector<path_elmt> vpe;
 parse_path(mp, path, vpe);
 return make_path(mp, &*vpe.begin(), &*vpe.end(), '/');
}

int main(int argc, char** argv)
{
 mem_pool mp;
 file out(mp, fd::out);
 file err(mp, fd::err);

 shException error(mp, 0);
 try {
     hard_assert(cnv(mp, "")=="");
     hard_assert(cnv(mp, "/")=="/");
     hard_assert(cnv(mp, "nm")=="nm");
     hard_assert(cnv(mp, "/nm")=="/nm");
     hard_assert(cnv(mp, "nm/")=="nm");
     hard_assert(cnv(mp, "/nm/")=="/nm");
     hard_assert(cnv(mp, "abc//ab///c////")=="abc/ab/c");
     hard_assert(cnv(mp, "////a///ab//abc/")=="/a/ab/abc");

     hard_assert(cnv(mp, "a:")=="a:");
     hard_assert(cnv(mp, "a:/")=="a:/");
     hard_assert(cnv(mp, "a:nm")=="a:nm");
     hard_assert(cnv(mp, "a:/nm")=="a:/nm");
     hard_assert(cnv(mp, "a:nm/")=="a:nm");
     hard_assert(cnv(mp, "a:/nm/")=="a:/nm");
     hard_assert(cnv(mp, "a:abc//ab///c////")=="a:abc/ab/c");
     hard_assert(cnv(mp, "a:////a///ab//abc/")=="a:/a/ab/abc");

     hard_assert(cnv(mp, "/")=="/");
     hard_assert(cnv(mp, "//")=="/");
     hard_assert(cnv(mp, "//a")=="/a");
     hard_assert(cnv(mp, "//a/")=="/a");
     hard_assert(cnv(mp, "//a//")=="/a");
     hard_assert(cnv(mp, "//a//b")=="/a/b");
     hard_assert(cnv(mp, "//a//b/")=="/a/b");
     hard_assert(cnv(mp, "//a//b//")=="/a/b");
     hard_assert(cnv(mp, "//a/b")=="//a/b");
     hard_assert(cnv(mp, "//a/b/")=="//a/b");
     hard_assert(cnv(mp, "//a/b//")=="//a/b");
     hard_assert(cnv(mp, "/a")=="/a");
     hard_assert(cnv(mp, "/a/")=="/a");
     hard_assert(cnv(mp, "/a//")=="/a");
     hard_assert(cnv(mp, "/a//b")=="/a/b");
     hard_assert(cnv(mp, "/a//b/")=="/a/b");
     hard_assert(cnv(mp, "/a//b//")=="/a/b");

     hard_assert(cnv(mp, "//srv/shr")=="//srv/shr");
     hard_assert(cnv(mp, "//srv/shr/")=="//srv/shr");
     hard_assert(cnv(mp, "//srv/shr/nm")=="//srv/shr/nm");
     hard_assert(cnv(mp, "//srv/shr/nm/")=="//srv/shr/nm");
     hard_assert(cnv(mp, "//srv/shr/abc//ab///c////")=="//srv/shr/abc/ab/c");
     hard_assert(cnv(mp, "//srv/shr/////a///ab//abc/")=="//srv/shr/a/ab/abc");

     hard_assert(get_path(mp, "")==normalize_path(mp, ""));
     hard_assert(get_name(mp, "")==normalize_path(mp, ""));
     hard_assert(get_path(mp, "/")==normalize_path(mp, "/"));
     hard_assert(get_name(mp, "/")==normalize_path(mp, ""));
     hard_assert(get_path(mp, "a")==normalize_path(mp, ""));
     hard_assert(get_name(mp, "a")==normalize_path(mp, "a"));
     hard_assert(get_path(mp, "/a")==normalize_path(mp, "/"));
     hard_assert(get_name(mp, "/a")==normalize_path(mp, "a"));
     hard_assert(get_path(mp, "/a/")==normalize_path(mp, "/"));
     hard_assert(get_name(mp, "/a/")==normalize_path(mp, "a"));
     hard_assert(get_path(mp, "/a/b")==normalize_path(mp, "/a"));
     hard_assert(get_name(mp, "/a/b")==normalize_path(mp, "b"));
     hard_assert(get_path(mp, "//a/b")==normalize_path(mp, "//a/b"));
     hard_assert(get_name(mp, "//a/b")==normalize_path(mp, ""));
     hard_assert(get_path(mp, "a:")==normalize_path(mp, "a:"));
     hard_assert(get_name(mp, "a:")==normalize_path(mp, ""));
     hard_assert(get_path(mp, "a:/")==normalize_path(mp, "a:/"));
     hard_assert(get_name(mp, "a:/")==normalize_path(mp, ""));
     hard_assert(get_path(mp, "a:b")==normalize_path(mp, "a:"));
     hard_assert(get_name(mp, "a:b")==normalize_path(mp, "b"));
     hard_assert(get_path(mp, "a:/b")==normalize_path(mp, "a:/"));
     hard_assert(get_name(mp, "a:/b")==normalize_path(mp, "b"));

     if (argc!=2) {
        err.write("tst_path path\n");
        return 1;
     }

     vector<path_elmt> vpe;
     parse_path(mp, argv[1], vpe);

     out.write(text_buf(mp)+argv[1]+'\n');
     out.write(text_buf(mp)+make_path(mp, &*vpe.begin(), &*vpe.end())+'\n');
     prn(out, vpe);

     return 0;
 }
 catch (shException she) { error=she; }
 catch (...) { error=recatchException(mp, _FLINE_); }

 err.write(toTextAll(error));
 return 1;
}

