/*
 * Copyright (C) Sergey P. Derevyago, 2008.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Internal wldcrd_mtchr test.
 */

#include <ders/file.hpp>
#include <ders/text_buf.hpp>
#include <ders/wldcrd_mtchr.hpp>

using namespace ders;

int main(int argc, char** argv)
{
 mem_pool mp;
 file out(mp, fd::out);
 file err(mp, fd::err);

 hard_assert( wldcrd_mtchr(mp, "").match(""));
 hard_assert( wldcrd_mtchr(mp, "").match("abc"));
 hard_assert( wldcrd_mtchr(mp, "*").match(""));
 hard_assert( wldcrd_mtchr(mp, "*").match("abc"));

 hard_assert(!wldcrd_mtchr(mp, "abc").match(""));
 hard_assert( wldcrd_mtchr(mp, "abc").match("abc"));

 hard_assert(!wldcrd_mtchr(mp, "*a").match("abc"));
 hard_assert( wldcrd_mtchr(mp, "*a*").match("abc"));
 hard_assert(!wldcrd_mtchr(mp, "a").match("abc"));
 hard_assert( wldcrd_mtchr(mp, "a*").match("abc"));

 hard_assert(!wldcrd_mtchr(mp, "*b").match("abc"));
 hard_assert( wldcrd_mtchr(mp, "*b*").match("abc"));
 hard_assert(!wldcrd_mtchr(mp, "b").match("abc"));
 hard_assert(!wldcrd_mtchr(mp, "b*").match("abc"));

 hard_assert( wldcrd_mtchr(mp, "*c").match("abc"));
 hard_assert( wldcrd_mtchr(mp, "*c*").match("abc"));
 hard_assert(!wldcrd_mtchr(mp, "c").match("abc"));
 hard_assert(!wldcrd_mtchr(mp, "c*").match("abc"));

 hard_assert(!wldcrd_mtchr(mp, "?").match("abc"));
 hard_assert(!wldcrd_mtchr(mp, "??").match("abc"));
 hard_assert( wldcrd_mtchr(mp, "???").match("abc"));
 hard_assert(!wldcrd_mtchr(mp, "????").match("abc"));

 hard_assert( wldcrd_mtchr(mp, "?bc").match("abc"));
 hard_assert( wldcrd_mtchr(mp, "a?c").match("abc"));
 hard_assert( wldcrd_mtchr(mp, "ab?").match("abc"));
 hard_assert( wldcrd_mtchr(mp, "??c").match("abc"));
 hard_assert( wldcrd_mtchr(mp, "?b?").match("abc"));
 hard_assert( wldcrd_mtchr(mp, "a??").match("abc"));

 hard_assert(!wldcrd_mtchr(mp, "a?c*").match("aabc"));
 hard_assert( wldcrd_mtchr(mp, "*a?c*").match("aabc"));
 hard_assert( wldcrd_mtchr(mp, "*a?c").match("aabc"));

 hard_assert(!wldcrd_mtchr(mp, "?a*").match("aba"));
 hard_assert( wldcrd_mtchr(mp, "*?a*").match("aba"));
 hard_assert( wldcrd_mtchr(mp, "*?a").match("aba"));

 hard_assert( wldcrd_mtchr(mp, "a*b*c").match("abc"));
 hard_assert(!wldcrd_mtchr(mp, "a*b*c").match("adc"));
 hard_assert( wldcrd_mtchr(mp, "a*b*c").match("aebec"));
 hard_assert(!wldcrd_mtchr(mp, "a*b*c").match("aedec"));

 hard_assert( wldcrd_mtchr(mp, "a*b*c*d").match("abcd"));
 hard_assert(!wldcrd_mtchr(mp, "a*b*c*d").match("abce"));
 hard_assert( wldcrd_mtchr(mp, "a*b*c*d").match("afbffcfffd"));
 hard_assert(!wldcrd_mtchr(mp, "a*b*c*d").match("afbffcfffe"));

 hard_assert(!wldcrd_mtchr(mp, "\\").match(""));
 hard_assert( wldcrd_mtchr(mp, "\\").match("\\"));
 hard_assert( wldcrd_mtchr(mp, "a\\").match("a\\"));
 hard_assert( wldcrd_mtchr(mp, "\\\\").match("\\"));
 hard_assert( wldcrd_mtchr(mp, "a\\\\").match("a\\"));
 hard_assert( wldcrd_mtchr(mp, "\\a").match("a"));
 hard_assert( wldcrd_mtchr(mp, "a\\b").match("ab"));
 hard_assert( wldcrd_mtchr(mp, "a\\\\b").match("a\\b"));

 if (argc!=3) {
    err.write("tst_wcm mask str\n");
    return 1;
 }

 out.write(text_buf(mp)+'('+argv[1]+")\n");
 out.write(text_buf(mp)+'('+argv[2]+")\n");
 out.write( (wldcrd_mtchr(mp, argv[1]).match(argv[2])) ? "true" : "false" );

 return 0;
}

