/*
 * Copyright (C) Sergey P. Derevyago, 2008.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Example of stl_allocator performance.
 */

#include <list>
#include <vector>
#include <stdio.h>
#include <time.h>
#include <ders/stl_alloc.hpp>
#include <ders/thread.hpp>

using namespace std;
using namespace ders;

const int N=1000;
const int M=10000;

void start_std(void*)
{
 list<int> lst;
 for (int i=0; i<N; i++) {
     for (int j=0; j<M; j++) lst.push_back(j);
     for (int j=0; j<M; j++) lst.pop_front();
 }
}

void start_ders(void*)
{
 mem_pool mp;
 stl_allocator<int> alloc(mp);

 list<int, stl_allocator<int> > lst(alloc);
 for (int i=0; i<N; i++) {
     for (int j=0; j<M; j++) lst.push_back(j);
     for (int j=0; j<M; j++) lst.pop_front();
 }
}

int main(int argc, char** argv)
{
 if (argc!=3) {
    m1:
    fprintf(stderr, "main num_threads std|ders");
    return 1;
 }

 int numThr=atoi(argv[1]);
 if ( !(numThr>=1 && numThr<=100) ) {
    fprintf(stderr, "num_threads must be in [1, 100]");
    return 1;
 }

 void (*start)(void*);
 if (strcmp(argv[2], "std")==0) start=start_std;
 else if (strcmp(argv[2], "ders")==0) start=start_ders;
 else goto m1;

 clock_t c1=clock();

 mem_pool mp;
 vector<sh_thread> vthr;
 for (int i=0; i<numThr; i++) vthr.push_back(new_thread(mp, start, 0));
 for (int i=0; i<numThr; i++) vthr[i]->join();

 clock_t c2=clock();
 printf("%d\t%d\t%s\n", numThr, int(c2-c1), argv[2]);

 return 0;
}

