/*
 * Copyright (C) Sergey P. Derevyago, 2008.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Implements message classes.
 */

#include "msg.hpp"

namespace mtcnvsrc {  // ::mtcnvsrc

using namespace ders;

bool MsgIO::read_from(data_in& in)
{
 rd=sh_ptr<Msg>(mp, 0);

 char ch;
 if (!in.read_char(ch)) return false;

 bool ret;
 switch (ch) {
        case Msg::FindFiles: {
             mp_newbuf<FindFilesMsg> buf(mp);
             rd.set(buf.rls(::new(buf.get()) FindFilesMsg(nt(mp))));

             FindFilesMsg& m=rd->to<FindFilesMsg>();
             ret=in.read_text(m.dirName); assert(ret);

             break;
        }
        case Msg::ConvFile: {
             mp_newbuf<ConvFileMsg> buf(mp);
             rd.set(buf.rls(::new(buf.get()) ConvFileMsg(nt(mp), 0)));

             ConvFileMsg& m=rd->to<ConvFileMsg>();
             ret=in.read_text(m.fileName); assert(ret);
             ret=in.read_ulong(m.fileSize); assert(ret);

             break;
        }
        default: assert(false);
 }

 return true;
}

void MsgIO::write_to(data_out& out) const
{
 char ch=wr->getType();
 out.write_char(ch);

 switch (ch) {
        case Msg::FindFiles: {
             const FindFilesMsg& m=wr->to<FindFilesMsg>();
             out.write_text(m.dirName);

             break;
        }
        case Msg::ConvFile: {
             const ConvFileMsg& m=wr->to<ConvFileMsg>();
             out.write_text(m.fileName);
             out.write_ulong(m.fileSize);

             break;
        }
        default: assert(false);
 }
}

sh_ptr<Msg> MsgIO::read()
{
 if ((rc=dq.read_obj(*this))!=data_queue::ok) rd=sh_ptr<Msg>(mp, 0);
 return rd;
}

bool MsgIO::write(const Msg& msg)
{
 wr=&msg;
 rc=dq.write_obj(*this);

 return rc==data_queue::ok;
}

}  // namespace ::mtcnvsrc

