/*
 * Copyright (C) Sergey P. Derevyago, 2008.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Defines message classes.
 */

#ifndef __MSG_HPP__
#define __MSG_HPP__

#include <ders/data_queue.hpp>

namespace mtcnvsrc {  // ::mtcnvsrc

using namespace ders;

class Msg : virtual public destroyable {
 public:
      enum Type { FindFiles, ConvFile };

      Type getType() const { return type; }
      template<class M> M& to();
      template<class M> const M& to() const;

 protected:
      Msg(Type t) : type(t) {}

 private:
      Msg(const Msg&);
      Msg& operator=(const Msg&);

      Type type;
};

struct FindFilesMsg : public Msg {
       sh_text dirName;

       FindFilesMsg(sh_text dn) : Msg(FindFiles), dirName(dn) {}

       virtual void destroy(mem_pool& mp2) { destroy_this(this, mp2); }
};

struct ConvFileMsg : public Msg {
       sh_text fileName;
       unsigned long fileSize;

       ConvFileMsg(sh_text fn, unsigned long fs) : Msg(ConvFile), fileName(fn),
         fileSize(fs) {}

       virtual void destroy(mem_pool& mp2) { destroy_this(this, mp2); }
};

#ifndef DERS_BUG_PRIVATE
class MsgIO : private object_io {
#else
class MsgIO : protected object_io {
#endif
      mem_pool& mp;
      data_queue& dq;
      sh_ptr<Msg> rd;
      const Msg* wr;
      data_queue::result rc;

      virtual bool read_from(data_in& in);
      virtual void write_to(data_out& out) const;

      MsgIO(const MsgIO&);
      MsgIO& operator=(const MsgIO&);

 public:
      MsgIO(mem_pool& m, data_queue& d) : mp(m), dq(d), rd(mp, 0),
        rc(data_queue::ok) {}

      virtual void destroy(mem_pool& mp2) { destroy_this(this, mp2); }

      sh_ptr<Msg> read();
      bool write(const Msg& msg);
      data_queue::result result() { return rc; }
};

template<class M>
inline M& Msg::to() { return static_cast<M&>(*this); }

template<class M>
inline const M& Msg::to() const { return static_cast<const M&>(*this); }

}  // namespace ::mtcnvsrc

#endif  // __MSG_HPP__

