/*
 * Copyright (C) Sergey P. Derevyago, 2008.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Defines struct MainTask and related.
 */

#ifndef __MAINTASK_HPP__
#define __MAINTASK_HPP__

#include <ders/buf_cache.hpp>
#include <ders/hash_vec.hpp>
#include <ders/task.hpp>
#include <ders/text_buf.hpp>
#include "msg.hpp"

namespace mtdirdiff {  // ::mtdirdiff

using namespace ders;
using namespace std;

struct DirCont {
       mem_pool ownmp;
       sh_text name;
       hash_vec<sh_text, char> dirs;
       hash_vec<sh_text, unsigned long> files;

       DirCont(const ch_rng& nm) : name(nt(ownmp, nm)), dirs(101), files(1001)
         {}
};

struct MainTask : public task {
       struct ErrData {
              text_buf& err;
              sh_text msg;

              ErrData(text_buf& e, sh_text m) : err(e), msg(m) {}
       };

       struct Stat {
              int addCnt;
              int delCnt;
              int modCnt;

              Stat() : addCnt(0), delCnt(0), modCnt(0) {}
       };

       struct Arg {
              buf_cache bc;
              Stat st;

              Arg() : bc(1024*1024) {}
       };

       mem_pool& gmp;
       text_buf gerr;
       int argno;
       vector<sh_ptr<Arg> > args;

       DirCont oldDir;
       DirCont newDir;
       sh_text diffDirAdd;
       sh_text diffDirDel;
       sh_text diffDirMod;

       MainTask(sh_text od, sh_text nd, sh_text dd);

       static void addError(void* arg);
       static sh_ptr<Stat> newStat(mem_pool& mp);
       static sh_ptr<Arg> newArg(mem_pool& mp);

       virtual void destroy(mem_pool& mp2) { destroy_this(this, mp2); }
       virtual void* proc_arg();
       virtual void proc(mem_pool& mp, const dq_vec& dqv, void* arg, task_opers&
         to);

       void doFindFiles(mem_pool& mp, data_queue& dq, Arg& arg, const
         FindFilesMsg& msg);
       void doCompare(mem_pool& mp, data_queue& dq, Arg& arg, const CompareMsg&
         msg);
       void doCopyFile(mem_pool& mp, data_queue& dq, Arg& arg, const
         CopyFileMsg& msg);
       void doCompareFile(mem_pool& mp, data_queue& dq, Arg& arg, const
         CompareFileMsg& msg);

       sh_ptr<Stat> getStat(mem_pool& mp);
};

}  // namespace ::mtdirdiff

#endif  // __MAINTASK_HPP__

