/*
 * Copyright (C) Sergey P. Derevyago, 2008.
 *
 * Permission to copy, use, modify, sell and distribute this software is granted
 * provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied warranty, and
 * with no claim as to its suitability for any purpose.
 *
 */

/** @file
 * Implements message classes.
 */

#include "msg.hpp"

namespace mtdirdiff {  // ::mtdirdiff

using namespace ders;

bool MsgIO::read_from(data_in& in)
{
 rd=sh_ptr<Msg>(mp, 0);

 char ch;
 if (!in.read_char(ch)) return false;

 bool ret;
 switch (ch) {
        case Msg::FindFiles: {
             mp_newbuf<FindFilesMsg> buf(mp);
             rd.set(buf.rls(::new(buf.get()) FindFilesMsg(0)));

             FindFilesMsg& m=rd->to<FindFilesMsg>();
             ret=in.read_ptr(m.dir); assert(ret);

             break;
        }
        case Msg::Compare: {
             mp_newbuf<CompareMsg> buf(mp);
             rd.set(buf.rls(::new(buf.get()) CompareMsg(false)));

             CompareMsg& m=rd->to<CompareMsg>();
             ret=in.read_bool(m.oldToNew); assert(ret);

             break;
        }
        case Msg::CopyFile: {
             mp_newbuf<CopyFileMsg> buf(mp);
             rd.set(buf.rls(::new(buf.get()) CopyFileMsg(nt(mp), nt(mp), 0)));

             CopyFileMsg& m=rd->to<CopyFileMsg>();
             ret=in.read_text(m.fromName); assert(ret);
             ret=in.read_text(m.toName); assert(ret);
             ret=in.read_char(m.dest); assert(ret);

             break;
        }
        case Msg::CompareFile: {
             mp_newbuf<CompareFileMsg> buf(mp);
             rd.set(buf.rls(::new(buf.get()) CompareFileMsg(0)));

             CompareFileMsg& m=rd->to<CompareFileMsg>();
             ret=in.read_int(m.oldPos); assert(ret);

             break;
        }
        default: assert(false);
 }

 return true;
}

void MsgIO::write_to(data_out& out) const
{
 char ch=wr->getType();
 out.write_char(ch);

 switch (ch) {
        case Msg::FindFiles: {
             const FindFilesMsg& m=wr->to<FindFilesMsg>();
             out.write_ptr(m.dir);

             break;
        }
        case Msg::Compare: {
             const CompareMsg& m=wr->to<CompareMsg>();
             out.write_bool(m.oldToNew);

             break;
        }
        case Msg::CopyFile: {
             const CopyFileMsg& m=wr->to<CopyFileMsg>();
             out.write_text(m.fromName);
             out.write_text(m.toName);
             out.write_char(m.dest);

             break;
        }
        case Msg::CompareFile: {
             const CompareFileMsg& m=wr->to<CompareFileMsg>();
             out.write_int(m.oldPos);

             break;
        }
        default: assert(false);
 }
}

sh_ptr<Msg> MsgIO::read()
{
 if ((rc=dq.read_obj(*this))!=data_queue::ok) rd=sh_ptr<Msg>(mp, 0);
 return rd;
}

bool MsgIO::write(const Msg& msg)
{
 wr=&msg;
 rc=dq.write_obj(*this);

 return rc==data_queue::ok;
}

}  // namespace ::mtdirdiff

